#!/usr/local/bin/perl -w
# $Id: jmarc.pl,v 1.1 2002/03/13 07:10:42 masao Exp $

# jmarc.pl: Japan/MARC 쥳ɲѤΥ饤֥ꡣ

# Copyright (C) 1998-2000 by Masao Takaku <masao@ulis.ac.jp>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# This file must be encoded in EUC-JP.

package jmarc;
use NKF;

### Global Variables
$RecordSep = '\x1d';
$FieldSep = '\x1e';
$SubFieldSep = '\x1f';
$LabelLength = 24;

$NEW = 'N';
$DELETE = 'D';
$CHANGE = 'C';

# ɤΤ褦ʥɤˤ뤫
# FIXME: ĹΥ޻(a^i^u^e^o^, etc.)̾Υե٥åȤ᤹
%GAIJI = ("\x2a\x23" => "#A", # 
	  "\x2a\x2f" => "#I", # 
	  "\x2a\x39" => "#U", # 
	  "\x2a\x2b" => "#E", # 
	  "\x2a\x34" => "#O", # 
	  "\x2a\x43" => "#a", # 
	  "\x2a\x56" => "#i", # 
	  "\x2a\x6c" => "#u", # 
	  "\x2a\x50" => "#e", # 
	  "\x2a\x5e" => "#o"  # 
	  # "\x22\x31" => "\x21\x21" # 
	  );

sub get_records($) {
    my ($cont) = @_;
    my @records = split(/$RecordSep/, $cont);
    return @records;
}

sub get_label($) {
    my ($rec) = @_;
    return ebcdic2ascii(substr($rec, 0, $LabelLength));
}

sub get_directory($) {
    my ($rec) = @_;
    my $baseaddr = get_baseaddr($rec);
    return ebcdic2ascii(substr($rec, $LabelLength, $baseaddr-1 - $LabelLength));
}

sub get_data($) {
    my ($rec) = @_;
    return substr($rec, get_baseaddr($rec));
}

sub get_status($) {
    my ($rec) = @_;
    my $label = get_label($rec);
    return substr($label, 5, 1);
}

sub get_baseaddr($) {
    my ($rec) = @_;
    my $label = get_label($rec);
    return int substr($label, 12, 5);
}

sub get_jpno($) {
    my ($rec) = @_;
    my $data = get_data($rec);
    return ebcdic2ascii(substr($data, 0, 8));
}

sub get_type($) {
    my ($rec) = @_;
    my $label = get_label($rec);
    return substr($label, 6, 1);
}

sub read_record($$) {
    my ($filename, $position) = @_;
    open(MARC, $filename) || die "open fail MARC ($filename): $!";

    my $record = undef;
    seek MARC, $position, 0;
    read MARC, $record, 5 ||
        die "read fail MARC ($filename, $position): $!";
    my $length = int jmarc::ebcdic2ascii($record);
    
    seek MARC, $position, 0;
    read MARC, $record, $length-1 ||
        die "read fail MARC ($filename, $position): $!";
    close MARC;
    return $record;
}

sub get_fields($) {
    my ($rec) = @_;
    my $directory = get_directory($rec);
    my $data = get_data($rec);
    my %fields = ();
    while ($directory =~ s/^(\d{3})(\d{4})(\d{5})//) {
	my $field_id = $1;
	my $field_len = int $2;
	my $field_start = int $3;
	# print "ID: $field_id, Length: $field_len, Offset: $field_start\n";
	
	my $field = substr($data, $field_start, $field_len-1);
	if ($field !~ /^$SubFieldSep/) {
	    $fields{$field_id} = ebcdic2ascii($field);
	    next;
	}
	my @subfields = ();
	while ($field =~ s/^$SubFieldSep(.)(...)(.)//) {
	    my $sub_id = ebcdic2ascii($1);
	    my $sub_len = int ebcdic2ascii($2);
	    my $mode = ebcdic2ascii($3);
	    # print "SubID: $sub_id, Length: $sub_len, Mode: $mode\n";
	    my $fdata = substr($field, 0, $sub_len);
	    if ($mode eq "1") {
		push @subfields, {$sub_id => ebcdic2ascii($fdata)};
	    } elsif ($mode eq "2") {
		push @subfields, {$sub_id => escape_kanji($fdata)};
	    }
	    $field = substr($field, $sub_len);
	}
	$fields{$field_id} = [@subfields];
    }
    return %fields;
}

sub as_text($) {
    my ($rec) = @_;
    my $label = jmarc::get_label($rec);
    my $text = '';
    $text = "$label\n\n";

    my %fields = jmarc::get_fields($rec);
    foreach my $fid (sort keys %fields) {
	if ($fid eq "001") {
	    $text .= "$fid  $fields{$fid}\n";
	    next;
	}
	foreach my $subfield (@{$fields{$fid}}) {
	    while (my ($sub_fid, $value) = each %{$subfield}) {
		$text .= "$fid \$$sub_fid $value\n";
	    }
	}
    }
    return $text;
}

# FIXME: ϥåǤġ
   %desc = ('status' => { 'N' => "쥳",
			  'D' => "쥳",
			  'C' => "쥳" },
	    'type' => { 'A' => "ǰʪ",
			'B' => "󴩹ʪ",
			'E' => "Ͽ޻",
			'G' => "",
			'H' => "ޥֻ",
			'I' => "Ͽ",
			'K' => "Ż߲",
			'L' => "ԥ塼ե",
			'T' => "о㳲ѻ" },
	    '001' => "쥳ɼֹ",
	    '010' => { 'A' => "ISBN" },
	    '020' => { 'A' => "̾",
		       'B' => "ֹ"},
	    '100' => { 'A' => "Ūǡ" },
	    '101' => { 'A' => "ƥȤθ",
		       'C' => "ʸθ" },
	    '251' => { 'A' => "ܥȥ",
		       'B' => "ȥϢ",
		       'D' => "ʤ",
		       'F' => "Ǥɽ",
		       'W' => "" },
	    '252' => { 'A' => "ܥȥ",
		       'B' => "ȥϢ",
		       'D' => "ʤ",
		       'F' => "Ǥɽ",
		       'W' => "" },
	    '253' => { 'A' => "ܥȥ",
		       'B' => "ȥϢ",
		       'D' => "ʤ",
		       'F' => "Ǥɽ",
		       'W' => "" },
	    '254' => { 'A' => "ܥȥ",
		       'B' => "ȥϢ",
		       'D' => "ʤ",
		       'F' => "Ǥɽ",
		       'W' => "" },
	    '255' => { 'A' => "ܥȥ",
		       'B' => "ȥϢ",
		       'D' => "ʤ",
		       'F' => "Ǥɽ",
		       'W' => "" },
	    '256' => { 'A' => "ܥȥ",
		       'B' => "ȥϢ",
		       'D' => "ʤ",
		       'F' => "Ǥɽ",
		       'W' => "" },
	    '257' => { 'A' => "ܥȥ",
		       'B' => "ȥϢ",
		       'D' => "ʤ",
		       'F' => "Ǥɽ",
		       'W' => "" },
	    '258' => { 'A' => "ܥȥ",
		       'B' => "ȥϢ",
		       'D' => "ʤ",
		       'F' => "Ǥɽ",
		       'W' => "" },
	    '259' => { 'A' => "ܥȥ",
		       'B' => "ȥϢ",
		       'D' => "ʤ",
		       'F' => "Ǥɽ",
		       'W' => "" },
	    '261' => { 'A' => "󥿥ȥ" },
	    '265' => { 'A' => "ɽʤ" },
	    '270' => { 'A' => "",
		       'B' => "Ǽ",
		       'D' => "ǯ" },
	    '275' => { 'A' => "ڡʤ",
		       'B' => "礭",
		       'E' => "°" },
	    '280' => { 'A' => "ѽ̾",
		       'B' => "ѽֹ",
		       'D' => "ѽ̾",
		       'F' => "ѽֹ" },
	    '281' => { 'A' => "ܥ꡼̾",
		       'B' => "꡼Ϣ",
		       'D' => "꡼ֹ",
		       'F' => "꡼˴ؤǤɽ",
		       'S' => "̥꡼̾",
		       'X' => "꡼ISSN" },
	    '282' => { 'A' => "ܥ꡼̾",
		       'B' => "꡼Ϣ",
		       'D' => "꡼ֹ",
		       'F' => "꡼˴ؤǤɽ",
		       'S' => "̥꡼̾",
		       'X' => "꡼ISSN" },
	    '283' => { 'A' => "ܥ꡼̾",
		       'B' => "꡼Ϣ",
		       'D' => "꡼ֹ",
		       'F' => "꡼˴ؤǤɽ",
		       'S' => "̥꡼̾",
		       'X' => "꡼ISSN" },
	    '291' => { 'A' => "ƴΥȥ",
		       'B' => "ƴΥȥϢ",
		       'D' => "ƴδʤ",
		       'F' => "ƴǤɽ" },
	    '291' => { 'A' => "ƴΥȥ",
		       'B' => "ƴΥȥϢ",
		       'D' => "ƴδʤ",
		       'F' => "ƴǤɽ" },
	    '292' => { 'A' => "ƴΥȥ",
		       'B' => "ƴΥȥϢ",
		       'D' => "ƴδʤ",
		       'F' => "ƴǤɽ" },
	    '293' => { 'A' => "ƴΥȥ",
		       'B' => "ƴΥȥϢ",
		       'D' => "ƴδʤ",
		       'F' => "ƴǤɽ" },
	    '294' => { 'A' => "ƴΥȥ",
		       'B' => "ƴΥȥϢ",
		       'D' => "ƴδʤ",
		       'F' => "ƴǤɽ" },
	    '295' => { 'A' => "ƴΥȥ",
		       'B' => "ƴΥȥϢ",
		       'D' => "ƴδʤ",
		       'F' => "ƴǤɽ" },
	    '296' => { 'A' => "ƴΥȥ",
		       'B' => "ƴΥȥϢ",
		       'D' => "ƴδʤ",
		       'F' => "ƴǤɽ" },
	    '297' => { 'A' => "ƴΥȥ",
		       'B' => "ƴΥȥϢ",
		       'D' => "ƴδʤ",
		       'F' => "ƴǤɽ" },
	    '298' => { 'A' => "ƴΥȥ",
		       'B' => "ƴΥȥϢ",
		       'D' => "ƴδʤ",
		       'F' => "ƴǤɽ" },
	    '299' => { 'A' => "ƴΥȥ",
		       'B' => "ƴΥȥϢ",
		       'D' => "ƴδʤ",
		       'F' => "ƴǤɽ" },
	    '350' => { 'A' => "" },
	    '354' => { 'A' => "θȥ" },
	    '360' => { 'A' => "",
		       'B' => "ǹ߲",
		       'C' => "β" },
	    '377' => { 'A' => "" },
	    '386' => { 'A' => "ե" },
	    '387' => { 'A' => "ƥ׷" },
	    '551' => { 'A' => "ȥɸ: ʷ",
		       'X' => "ȥɸ: ޻",
		       'B' => "ȥɸ: (ߥե)",
		       'D' => "ȥɸ: ʤɤɤ" },
	    '552' => { 'A' => "ȥɸ: ʷ",
		       'X' => "ȥɸ: ޻",
		       'B' => "ȥɸ: (ߥե)",
		       'D' => "ȥɸ: ʤɤɤ" },
	    '553' => { 'A' => "ȥɸ: ʷ",
		       'X' => "ȥɸ: ޻",
		       'B' => "ȥɸ: (ߥե)",
		       'D' => "ȥɸ: ʤɤɤ" },
	    '554' => { 'A' => "ȥɸ: ʷ",
		       'X' => "ȥɸ: ޻",
		       'B' => "ȥɸ: (ߥե)",
		       'D' => "ȥɸ: ʤɤɤ" },
	    '555' => { 'A' => "ȥɸ: ʷ",
		       'X' => "ȥɸ: ޻",
		       'B' => "ȥɸ: (ߥե)",
		       'D' => "ȥɸ: ʤɤɤ" },
	    '556' => { 'A' => "ȥɸ: ʷ",
		       'X' => "ȥɸ: ޻",
		       'B' => "ȥɸ: (ߥե)",
		       'D' => "ȥɸ: ʤɤɤ" },
	    '557' => { 'A' => "ȥɸ: ʷ",
		       'X' => "ȥɸ: ޻",
		       'B' => "ȥɸ: (ߥե)",
		       'D' => "ȥɸ: ʤɤɤ" },
	    '558' => { 'A' => "ȥɸ: ʷ",
		       'X' => "ȥɸ: ޻",
		       'B' => "ȥɸ: (ߥե)",
		       'D' => "ȥɸ: ʤɤɤ" },
	    '559' => { 'A' => "ȥɸ: ʷ",
		       'X' => "ȥɸ: ޻",
		       'B' => "ȥɸ: (ߥե)",
		       'D' => "ȥɸ: ʤɤɤ" },
	    '580' => { 'A' => "ѽ̾ɸ: ʷ",
		       'B' => "ѽ̾ɸ: (ߥե)",
		       'X' => "ѽ̾ɸ: ޻",
		       'D' => "ѽֹɤ" },
	    '581' => { 'A' => "꡼̾ɸ: ʷ",
		       'B' => "꡼̾ɸ: (ߥե)",
		       'X' => "꡼̾ɸ: ޻",
		       'D' => "꡼̾ɸ: ʤɤɤ" },
	    '582' => { 'A' => "꡼̾ɸ: ʷ",
		       'B' => "꡼̾ɸ: (ߥե)",
		       'X' => "꡼̾ɸ: ޻",
		       'D' => "꡼̾ɸ: ʤɤɤ" },
	    '583' => { 'A' => "꡼̾ɸ: ʷ",
		       'B' => "꡼̾ɸ: (ߥե)",
		       'X' => "꡼̾ɸ: ޻",
		       'D' => "꡼̾ɸ: ʤɤɤ" },
	    '591' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: (ߥե)",
		       'X' => "ƴɸ: ޻",
		       'D' => "ƴɸ: ʤɤɤ" },
	    '592' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: (ߥե)",
		       'X' => "ƴɸ: ޻",
		       'D' => "ƴɸ: ʤɤɤ" },
	    '593' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: (ߥե)",
		       'X' => "ƴɸ: ޻",
		       'D' => "ƴɸ: ʤɤɤ" },
	    '594' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: (ߥե)",
		       'X' => "ƴɸ: ޻",
		       'D' => "ƴɸ: ʤɤɤ" },
	    '595' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: (ߥե)",
		       'X' => "ƴɸ: ޻",
		       'D' => "ƴɸ: ʤɤɤ" },
	    '596' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: (ߥե)",
		       'X' => "ƴɸ: ޻",
		       'D' => "ƴɸ: ʤɤɤ" },
	    '597' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: (ߥե)",
		       'X' => "ƴɸ: ޻",
		       'D' => "ƴɸ: ʤɤɤ" },
	    '598' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: (ߥե)",
		       'X' => "ƴɸ: ޻",
		       'D' => "ƴɸ: ʤɤɤ" },
	    '599' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: (ߥե)",
		       'X' => "ƴɸ: ޻",
		       'D' => "ƴɸ: ʤɤɤ" },
	    '650' => { 'A' => "Ŀͷ̾ɸ: ʷ",
		       'B' => "Ŀͷ̾ɸ: ",
		       'X' => "Ŀͷ̾ɸ: ޻" },
	    '658' => { 'A' => "̷̾ɸ: ʷ",
		       'B' => "̷̾ɸ: ",
		       'X' => "̷̾ɸ: ޻" },
	    '677' => { 'A' => "NDCʬ൭",
		       'V' => "NDCǼ" },
	    '685' => { 'A' => "NDLʬ൭",
		       'X' => "޻ʬ൭" },
	    '751' => { 'A' => "ɸ: ʷ",
		       'B' => "ɸ: ",
		       'X' => "ɸ: ޻" },
	    '752' => { 'A' => "ɸ: ʷ",
		       'B' => "ɸ: ",
		       'X' => "ɸ: ޻" },
	    '753' => { 'A' => "ɸ: ʷ",
		       'B' => "ɸ: ",
		       'X' => "ɸ: ޻" },
	    '754' => { 'A' => "ɸ: ʷ",
		       'B' => "ɸ: ",
		       'X' => "ɸ: ޻" },
	    '755' => { 'A' => "ɸ: ʷ",
		       'B' => "ɸ: ",
		       'X' => "ɸ: ޻" },
	    '756' => { 'A' => "ɸ: ʷ",
		       'B' => "ɸ: ",
		       'X' => "ɸ: ޻" },
	    '757' => { 'A' => "ɸ: ʷ",
		       'B' => "ɸ: ",
		       'X' => "ɸ: ޻" },
	    '758' => { 'A' => "ɸ: ʷ",
		       'B' => "ɸ: ",
		       'X' => "ɸ: ޻" },
	    '759' => { 'A' => "ɸ: ʷ",
		       'B' => "ɸ: ",
		       'X' => "ɸ: ޻" },
	    '781' => { 'A' => "꡼ɸ: ʷ",
		       'B' => "꡼ɸ: ",
		       'X' => "꡼ɸ: ޻" },
	    '782' => { 'A' => "꡼ɸ: ʷ",
		       'B' => "꡼ɸ: ",
		       'X' => "꡼ɸ: ޻" },
	    '783' => { 'A' => "꡼ɸ: ʷ",
		       'B' => "꡼ɸ: ",
		       'X' => "꡼ɸ: ޻" },
	    '791' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: ",
		       'X' => "ƴɸ: ޻" },
	    '792' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: ",
		       'X' => "ƴɸ: ޻" },
	    '793' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: ",
		       'X' => "ƴɸ: ޻" },
	    '794' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: ",
		       'X' => "ƴɸ: ޻" },
	    '795' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: ",
		       'X' => "ƴɸ: ޻" },
	    '796' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: ",
		       'X' => "ƴɸ: ޻" },
	    '797' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: ",
		       'X' => "ƴɸ: ޻" },
	    '798' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: ",
		       'X' => "ƴɸ: ޻" },
	    '799' => { 'A' => "ƴɸ: ʷ",
		       'B' => "ƴɸ: ",
		       'X' => "ƴɸ: ޻" },
	    '905' => { 'A' => "NDLֹ" },
	    '906' => { 'A' => "NDLΰֹ" }
	    );

sub as_html($$) {
    my ($rec, $display) = @_;
    my $html = '';
    my $status = jmarc::get_status($rec);
    my $type = jmarc::get_type($rec);
    my $jpno = jmarc::get_jpno($rec);
    my %fields = jmarc::get_fields($rec);

    my $bgcolor = '#ffff00';
    $html = "<p>". $desc{'status'}{$status} .": JP$jpno</p>\n";
    if ($display =~ /full/i) {
	$html .= "<table>\n";
	$html .= "<tr><td bgcolor=\"$bgcolor\">쥳ɼ</td><td>". $desc{'type'}{$type} ."</td></tr>\n";
	foreach my $fid (sort keys %fields) {
	    if ($fid eq "001") {
		$html .= "<tr><td bgcolor=\"$bgcolor\">$desc{'001'}</td><td>$fields{$fid}</td>\n";
		next;
	    }
	    foreach my $subfield (@{$fields{$fid}}) {
		while (my ($sub_fid, $value) = each %{$subfield}) {
		    $html .= "<tr><td bgcolor=\"$bgcolor\">";
		    $html .= $desc{$fid}{$sub_fid};
		    $html .= "</td><td>". nkf('-e', $value) ."</td></tr>\n";
		}
	    }
	}
	$html .= "</table>\n";
    } else {
	$html .= "<dl><dt><font size=\"+1\">";
	foreach my $field251 (@{$fields{'251'}}) {
	    if (defined $field251->{'A'}) {
		$html .= nkf('-e', $field251->{'A'});
		last;
	    }
	}
	$html .= "</font>\n";
	$html .= "<dd>";
	foreach my $field251 (@{$fields{'251'}}) {
	    if (defined $field251->{'F'}) {
		$html .= nkf('-e',$field251->{'F'}) . "\n";
		last;
	    }
	}
	foreach my $field270 (@{$fields{'270'}}) {
	    if (defined $field270->{'B'}) {
		$html .= "(" . nkf('-e', $field270->{'B'}) . ")";
		last;
	    }
	}
	$html .= "</dl>\n";
    }
    return $html;
}

# JIS X 0208 򥨥ץɤޤ֤
# FIXME: ƱˡνԤʤ
sub escape_kanji($) {
    my ($data) = @_;

    my $str = '';
    while (length($data)) {
        die "Length is Odd.\n" if (length($data) == 1);
        my $kanji = substr($data, 0, 2);
        $data = substr($data, 2);
	$kanji = $GAIJI{$kanji} if defined $GAIJI{$kanji};

	# FIXME: ¾γѶˤ롣
        $kanji =~ s/[\x30-\x7e][\xa1-\xfe]/\x21\x21/;
        $kanji =~ s/[\x29-\x2f][\x21-\x7e]/\x21\x21/;
        $kanji =~ s/[\x22][\x2f-\x68]/\x21\x21/;

        $str .= $kanji;
    }

    # ľʸ(or ʿ̾)ξ硢
    ## : ǡ١ä١ etc.
    # ޥʥ(: \x21\x5d)Ĺ(: \x21\x3c)᤹
    $str =~ s/([\x24-\x25][\x21-\x76])\x21\x5d/$1\x21\x3c/g;

    # JIS X 0207 ʹʸˤʤɤؤб ?
    # FIXME: 楳ɤƺƤ
    $str =~ s/[\x1c][\x4e-\x53]//g;
    
    # FIXME: JIS X 0208-1978 ( JIS C 6328 97 ) escape sequence.
    return "\x1b\x24\x40$str\x1b\x28\x42";
}
 
my @e2a_table = (
'\000', '\001', '\002', '\003', '\004', '\005', '\006', '\007', 
'\010', '\011', '\012', '\013', '\014', '\015', '\016', '\017', 
'\020', '\021', '\022', '\023', '\024', '\025', '\026', '\027', 
'\030', '\031', '\032', '\033', '\034', '\035', '\036', '\037', 
'\040', '\041', '\042', '\043', '\044', '\045', '\046', '\047', 
'\050', '\051', '\052', '\053', '\054', '\055', '\056', '\057', 
'\060', '\061', '\062', '\063', '\064', '\065', '\066', '\067', 
'\070', '\071', '\072', '\073', '\074', '\075', '\076', '\077', 
' ',    '\101', '\102', '\103', '\104', '\105', '\106', '\107', 
'\110', '\111', '[',    '.',    '<',    '(',    '+',    '!', 
'&',    '\121', '\122', '\123', '\124', '\125', '\126', '\127', 
'\130', '\131', ']',    '\\',    '*',    ')',    ';',    '^', 
'-',    '/',    '\142', '\143', '\144', '\145', '\146', '\147', 
'\150', '\151', '|',    ',',    '%',    '_',    '>',    '?', 
'\160', '\161', '\162', '\163', '\164', '\165', '\166', '\167', 
'\170', '\'',   ':',    '#',    '@',    '`',    '=',    '"', 
'\200', 'a',    'b',    'c',    'd',    'e',    'f',    'g', 
'h',    'i',    '\212', '\213', '\214', '\215', '\216', '\217', 
'\220', 'j',    'k',    'l',    'm',    'n',    'o',    'p', 
'q',    'r',    '\232', '\233', '\234', '\235', '\236', '\237', 
'\240', '-',    's',    't',    'u',    'v',    'x',    'w', 
'y',    'z',    '\252', '\253', '\254', '\255', '\256', '\257', 
'\260', '\261', '\262', '\263', '\264', '\265', '\266', '\267', 
'\270', '\271', '\272', '\273', '\274', '\275', '\276', '\277', 
'{',    'A',    'B',    'C',    'D',    'E',    'F',    'G', 
'H',    'I',    '\312', '\313', '\314', '\315', '\316', '\317', 
'}',    'J',    'K',    'L',    'M',    'N',    'O',    'P', 
'Q',    'R',    '\332', '\333', '\334', '\335', '\336', '\337', 
'$',    '\341', 'S',    'T',    'U',    'V',    'W',    'X', 
'Y',    'Z',    '\352', '\353', '\354', '\355', '\356', '\357', 
'0',    '1',    '2',    '3',    '4',    '5',    '6',    '7', 
'8',    '9',    '\372', '\373', '\374', '\375', '\376', '\377'
);

sub ebcdic2ascii($) {
    my ($e) = @_;
    
    $e =~ s/([\000-\377])/$e2a_table[ord($1)]/g;

    return $e;
}

1;

__END__

# ʲϴñʥޥ˥奢Ǥ

=head1 NAME

jmarc.pl - Japan/MARC library for perl

=head1 SYSNOPSYS

  require 'jmarc.pl';
  
  my @records = jmarc::get_records($contents);

  foreach my $record (@records) {
      next if jmarc::get_status($record) eq $jmarc::DELETE;
      my %fields = jmarc::get_fields($record);

      foreach my $fid (sort keys %fields) {
	  if ($fid eq "001") {
	      print "$fid  $fields{$fid}\n";
	      next;
	  }
	  foreach my $subfield (@{$fields{$fid}}) {
	      while (my ($sub_fid, $value) = each %{$subfield}) {
		  print "$fid \$$sub_fid $value\n";
	      }
	  }
      }
      print "\n";
  }

=head1 DESCRIPTION

Υ⥸塼ϡΩ޽ۤۤƤJapan/MARC쥳ɤ
뤿Υ饤֥Ǥ롣

=over 4

=item jmarc::get_records()

ޤޤƤʣΥ쥳ɤꥹȤ֤

=item jmarc::read_record($file, $pos)

ե̾ȳϰ֤򸵤˰쥳ɤɤ߹֤

=item jmarc::get_label()

쥳ɤΥ٥ʬ(Ĺ:24byte)֤

=item jmarc::get_directory()

ǥ쥯ȥʬ֤

=item jmarc::get_data()

ºݤνǡʬ֤

=item jmarc::get_status()

٥Τ쥳ɤξ()֤

=item jmarc::get_baseaddr()

ǡγϰ֤֤

=item jmarc::get_jpno()

JPֹ֤

=item jmarc::get_type()

쥳ɤμ̡ʰʪԥ塼եʤɡˤ֤

=item jmarc::get_fields()

쥳ɤΥեɹ¤ϤơΤϥå֤

=item jmarc::as_html()


=back

ʲϡʸѴǽĴؿǤ롣

=over 4

=item jmarc::escape_kanji()

2Х򳰻θISO-2022-JPʸѴ֤

=item jmarc::ebcdic2ascii()

EBCDICɤASCIIѴ롣

=back

=head1 COPYRIGHT

Copyright (C) 2000 Masao Takaku <masao@ulis.ac.jp>.
All rights reserved.

=cut
